@@ -53,6 +53,14 @@ class PluginEntryPoint(object):
         """Description with name. Handy for UI."""
         return "{0} ({1})".format(self.description, self.name)
 
+    @property
+    def long_description(self):
+        """Long description of the plugin."""
+        try:
+            return self.plugin_cls.long_description
+        except AttributeError:
+            return self.description
+
     @property
     def hidden(self):
         """Should this plugin be hidden from UI?"""
@@ -71,7 +79,7 @@ class PluginEntryPoint(object):
         return self._initialized is not None
 
     def init(self, config=None):
-        """Memoized plugin inititialization."""
+        """Memoized plugin initialization."""
         if not self.initialized:
             self.entry_point.require()  # fetch extras!
             self._initialized = self.plugin_cls(config, self.name)
@@ -222,7 +230,7 @@ class PluginsRegistry(collections.Mapping):
     def available(self):
         """Filter plugins based on availability."""
         return self.filter(lambda p_ep: p_ep.available)
-        # succefully prepared + misconfigured
+        # successfully prepared + misconfigured
 
     def find_init(self, plugin):
         """Find an initialized plugin.
